/*
 * file CqAttachment.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM 
 *
 * com.ibm.rational.wvcm.stp.cq.CqAttachment
 *
 * (C) Copyright IBM Corporation 2004, 2011.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.rational.wvcm.stp.cq;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import java.util.List;

import javax.wvcm.Feedback;
import javax.wvcm.Resource;
import javax.wvcm.WvcmException;
import javax.wvcm.PropertyNameList.PropertyName;

import com.ibm.rational.wvcm.stp.StpProperty;
import com.ibm.rational.wvcm.stp.StpResource;


/**
 * A proxy for an attachment resource.
 * 
 * <p>
 * Attachments are CqContextResources. Creation and modification of the
 * attachment takes place in the client's change context for the database. Since
 * to create, modify, or delete an action the host record must be modified an
 * action is needed to perform these operations. This action, however, applies
 * to the record with which the attachment is associated and not to the
 * attachment itself.
 * </p>
 * The user-friendly specification for the location of an attachment has the
 * form
 * 
 * <pre>
 *  <b>cq.record:</b><i>&lt;record-type&gt;</i>/<i>&lt;record-id&gt;</i>/<i>&lt;field-name&gt;</i>/<i>&lt;attachment-name&gt;</i><b>@</b><i>&lt;db-set&gt;</i>/<i>&lt;user-db&gt;</i>
 * </pre>
 * 
 * <p>
 * The parent location of an attachment is an attachment folder whose name is
 * the same as the name of the field with which the attachment is associated.
 * The parent location of the attachment folder is the record with which the
 * attachment is associated. Note that the attachment folder is also the value
 * of the attachment field.
 * <p>
 * The content of the attachment is accessed using the
 * {@link #doReadContent doReadContent()} method.
 * </p>
 */
public interface CqAttachment
    extends CqContextResource
{
    /**
     * Returns the attachment FILE_SIZE
     * @see javax.wvcm.Resource#getContentLength()
     */
    long getContentLength() throws WvcmException;
    
    /**
     * Returns the attachment DESCRIPTION
     * @see javax.wvcm.Resource#getComment()
     */
    String getComment() throws WvcmException;
    
    /** The unique key used to identify the attachment */
    PropertyName<String> DISPLAY_NAME = StpResource.DISPLAY_NAME;

    /**
     * Returns the value of the {@link #DISPLAY_NAME DISPLAY_NAME} property as
     * defined by this proxy.
     *
     * @return  A String containing the unique key used to identify this
     *          attachment
     *
     * @throws  WvcmException  if this proxy does not define a value for the
     *                         {@link #DISPLAY_NAME DISPLAY_NAME} property.
     */
    public String getDisplayName()
        throws WvcmException;

    /** The description associated with the attachment */
    PropertyName<String> DESCRIPTION =
        new PropertyName<String>(PROPERTY_NAMESPACE, "description");

    /**
     * Returns the value of the {@link #DESCRIPTION DESCRIPTION} property as
     * defined by this proxy.
     *
     * @return  A String containing a short description of this attachment.
     *
     * @throws  WvcmException  if this proxy does not define a value for the
     *                         {@link #DESCRIPTION DESCRIPTION} property.
     */
    public String getDescription()
        throws WvcmException;

    /**
     * Defines a new value for the DESCRIPTION property. Changes do not take
     * effect until doWriteProperties() has been successfully executed on this
     * proxy and the changes are delivered to the database.
     *
     * @param  description  The descriptive text to be associated with the
     *                      attachment.
     */
    public void setDescription(String description);

    /** The file name associated with the attachment */
    PropertyName<String> FILE_NAME =
        new PropertyName<String>(PROPERTY_NAMESPACE, "file-name");

    /**
     * Returns the value of the {@link #FILE_NAME FILE_NAME} property as defined
     * by this proxy.
     *
     * @return  A String containing the name of the file that originally held
     *          this attachment data.
     *
     * @throws  WvcmException  if this proxy does not define a value for the
     *                         {@link #FILE_NAME FILE_NAME} property.
     */
    public String getFileName()
        throws WvcmException;

    /** The size of the attachment in bytes */
    PropertyName<Long> FILE_SIZE =
        new PropertyName<Long>(PROPERTY_NAMESPACE, "file-size");

    /**
     * Returns the value of the {@link #FILE_SIZE FILE_SIZE} property as defined
     * by this proxy.
     *
     * @return  The number of bytes in this attachment.
     *
     * @throws  WvcmException  if this proxy does not define a value for the
     *                         {@link #FILE_SIZE FILE_SIZE} property.
     */
    public long getFileSize()
        throws WvcmException;

    /** The field (within a record) that references this attachment */
    PropertyName<CqFieldValue<?>> FIELD =
        new PropertyName<CqFieldValue<?>>(PROPERTY_NAMESPACE, "field");

    /**
     * Returns the value of the {@link #FIELD FIELD} property as defined by this
     * proxy.
     * 
     * @return A CqFieldValue identifying the field of which this attachment is
     *         a nested value.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #FIELD FIELD} property.
     */
    public CqFieldValue<?> getField()
        throws WvcmException;

    /**
     * The action under which this attachment's record is being modified; null
     * if the record is not being modified.
     */
    PropertyName<CqAction> ACTION = CqRecord.ACTION;

    /**
     * Returns the value of the {@link #ACTION ACTION} property as defined by
     * this proxy.
     * 
     * @return Returns an CqAction proxy representing the action under which the
     *         this attachment's record is being modified.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #ACTION ACTION} property.
     */
    CqAction getAction() throws WvcmException;

    /**
     * Defines a new value for the {@link #ACTION ACTION} property of this
     * proxy. Set this value prior to invoking doCreateAttachment,
     * doWriteProperties or doUnbindAll on this proxy to specify an action other
     * than the default action specified for those operations.
     * 
     * @param action A CqAction proxy identifying the action under which this
     *            attachment's record is to be modified when creating, updating,
     *            or deleting this attachment. For the update to succeed, the
     *            given action must be on the LEGAL_ACTIONs list for the
     *            attachment's record. If no value is provided, the action used
     *            to modify the attachment's parent record will be the default
     *            MODIFY type action if that is uniquely defined by the record
     *            type.
     * 
     * @return this proxy
     */
    CqAttachment setAction(CqAction action);
    
    /**
     * Creates a new attachment at the location specified by this proxy(*) and
     * initializes its content from a named file.
     * 
     * <p>
     * (*) The server is permitted to modify the location. The proxy returned is
     * a proxy that addresses the location actually chosen by the server. For
     * attachments, the location must be in the form
     * </p>
     * <p>
     * 
     * <pre>
       cq.record:&lt;record-type>/&lt;record-id>/&lt;field-name>/&lt;file-name>&#64;&lt;db-set>/&lt;user-db>
     * </pre>
     * 
     * <p>
     * where
     * &lt;record-type>/&lt;record-id>/&lt;field-name>/&lt;file-name>&#64;&lt;db-set>/&lt;user-db>
     * identifies the field with which this new attachment is to be associated
     * <p>
     * To create an attachment, the host record must be modified. If the record
     * is not already editable, the action under which the record should be
     * modified can be specified as the value of the ACTION property of this
     * proxy. If no action is specified and the record is not already being
     * modified, the default modify action will be used, if unique.
     * 
     * @param filename The full file pathname of the file from which the content
     *            of the attachment is to be read. The file named by this
     *            parameter must exist until the change context of this proxy is
     *            delivered to the database.
     * @param feedback A Feedback instance requesting the properties desired
     *            from the created attachment and related resources. May be
     *            <b>null</b> if no properties are desired.
     * @param deliveryOrder If {@link CqProvider#HOLD} the new attachment and
     *            its modified record are left in a writable state in the target
     *            database's change context. The change context must be
     *            delivered before the changes become visible outside the change
     *            context. To deliver the entire change context in an arbitrary
     *            order, use {@link CqProvider#DELIVER_ALL}. To deliver just
     *            the new attachment and its modified record, use
     *            {@link CqProvider#DELIVER}. If not one of these values, the
     *            modified resource named in this argument will be delivered to
     *            the database in the order specified in this argument.
     * 
     * @return An CqAttachment proxy for the created attachment. This proxy will
     *         define the properties requested by the Feedback argument.
     * 
     * @throws WvcmException If the resource cannot be created with the given
     *             properties. If the resource is successfully created, but
     *             properties cannot be written to it or the changes cannot be
     *             delivered to the database (when requested), the resource
     *             attribute of the thrown exception will be a proxy for the
     *             newly created resource. If the new resource could not be
     *             created, the resource attribute of the exception will be this
     *             proxy.
     */
    CqAttachment doCreateAttachment(String filename,
                                    Feedback feedback,
                                    List<CqContextResource> deliveryOrder)
        throws WvcmException;
    
    /**
     * Updating an attachment requires that the parent record be opened for 
     * update. If an action has not already been started on the parent record,
     * the action to use may be specified by setting the ACTION property of this
     * proxy before invoking this method.
     * 
     * @see CqContextResource#doWriteProperties(Feedback)
     */
    Resource doWriteProperties(Feedback feedback)
        throws WvcmException;

    /**
     * Updating an attachment requires that the parent record be opened for
     * update. If an action has not already been started on the parent record,
     * the action to use may be specified by setting the ACTION property of this
     * proxy before invoking this method.
     * 
     * Note that use of the {@link CqProvider#DELIVER} delivery order list
     * requests delivery of this attachment, its parent record, and any other
     * attachments currently in the change context.
     * 
     * @see CqContextResource#doWriteProperties(Feedback, List)
     */
    CqContextResource doWriteProperties(
        Feedback feedback,
        List<CqContextResource>  deliveryOrder)
        throws WvcmException;

    /**
     * Deleting an attachment requires that the parent record be opened for
     * update. If an action has not already been started on the parent record,
     * the action to use may be specified by setting the ACTION property of this
     * proxy before invoking this method.
     * 
     * @see CqContextResource#doUnbindAll(Feedback)
     */
    void doUnbindAll(Feedback feedback)
        throws WvcmException;

    /**
     * Deleting an attachment requires that the parent record be opened for
     * update. If an action has not already been started on the parent record,
     * the action to use may be specified by setting the ACTION property of this
     * proxy before invoking this method.
     * 
     * Note that use of the {@link CqProvider#DELIVER} delivery order list
     * requests deletion of this attachment and delivery of its parent record
     * and any other attachments currently in the change context.
     * 
     * @see CqContextResource#doUnbindAll(Feedback, List)
     */
    void 
    doUnbindAll(Feedback feedback, 
             List<CqContextResource> deliveryOrder)
        throws WvcmException;

    /**
     * Gets both the content and selected properties of this attachment.
     * 
     * <p>
     * This version of doReadContent is more efficient than the standard WVCM
     * doReadContent when interfacing with the ClearQuest desktop application
     * since the application can write the attachment contents directly to the
     * named file without the need to create a temporary file.
     * </p>
     * 
     * @param filename the resource content is written to the file named by this
     *            string.
     * @param feedback A request for the properties to be returned in the result
     *            proxy. May be <b>null</b>.
     * 
     * @return a CqAttachment proxy containing the requested properties.
     * 
     * @throws WvcmException if problems are encountered communicating with
     *             server
     */
    public CqAttachment doReadContent(
        String   filename,
        Feedback feedback)
        throws WvcmException;
}
